@@ -151,6 +151,9 @@ struct _AppUIData {
     /* Switch to page dialog */
     GtkWidget *switch_page_dialog;
 
+    /* Open-save dialog */
+    GtkWidget *opensave_dialog;
+
     /* The last pointer position where it was updated */
     gint lastx;
     gint lasty;
@@ -189,6 +192,10 @@ struct _AppUIData {
     
     /* This is used to workaround other bugs */
     guint key_pressed;
+
+    gboolean save_dialog_opened;
+
+    gchar *last_uri;
 };
 
 #endif /* APPDATA_H */
