@@ -408,9 +408,14 @@ render_page_func(gpointer data)
 
     TDB("render_page_func begin\n");
     priv->cancel_render = FALSE;
-    
+
     // If displaying page fails show empty page
     try {
+        if (get_free_space() == 0) {
+            // rendering with full storage causes crash, avoid it
+            g_warning( "Not enough memory on flash." );
+            throw (0);
+        }
     	display_page();
     } catch( int e ) {
     	fprintf( stderr, "%s: Can't display page\n", __FUNCTION__ );
@@ -429,7 +434,7 @@ render_page_func(gpointer data)
         priv->render_thread = NULL;      
         return NULL;
     }
-    	
+ 	
     if (!priv->cancel_render)
     {
 		DTRY(gdk);
@@ -1404,6 +1409,24 @@ pdf_viewer_empty_document()
     priv->current_page = PDF_PAGE_INIT;
     priv->x = priv->y = 0;
 
+    if (priv->write_handle)
+    {
+        gnome_vfs_close(priv->write_handle);
+        priv->write_handle= NULL;
+    }
+
+    if (priv->read_handle)
+    {
+        gnome_vfs_close(priv->read_handle);
+        priv->read_handle = NULL;
+    }
+
+    if (priv->vfs_handle)
+    {
+        gnome_vfs_close(priv->vfs_handle);
+        priv->vfs_handle = NULL;
+    }
+
 	/* g_debug( "%s 5", __FUNCTION__ ); */
     ui_enable_document_open(priv->app_ui_data, TRUE);
 
@@ -1561,7 +1584,8 @@ pdf_viewer_open(const char *uri, const char *password)
 
         /* If opening for gateway/shared, copy to tmp first */
         if (g_str_has_prefix(uri, "obex://") ||
-            g_str_has_prefix(uri, "upnpav://") ) {
+            g_str_has_prefix(uri, "upnpav://") ||
+            g_str_has_prefix(uri, "smb://")) {
             
             g_debug( "%s, Make local copy out of remote file", __FUNCTION__ );
             priv->is_gateway = TRUE;
@@ -2522,13 +2546,9 @@ scrollbar_change_idle(gpointer app_ui_data)
 	NOTE Caller must free allocated memory using g_free
 	when non-NULL value is returned.
 
-	@TODO value string copy not tested with unicode values, may generate
-	warnings: "WARNING **: Invalid UTF8 string passed to
-	pango_layout_set_text()"
-	@see printInfoString() in xpdf/pdfinfo.cc
-	
 	@param key key to document information dictionary
-	@return value corresponding to the key; NULL if error or not found
+	@return value corresponding to the key in UTF8; 
+                NULL if error or not found
 */
 gchar *
 pdf_viewer_get_info(char *key)
@@ -2547,7 +2567,30 @@ pdf_viewer_get_info(char *key)
 
     /* get value for specified key from the dictionary */
     if (info.isDict() && (info.getDict()->lookup(key, &obj)->isString()))
-        value = g_strdup(obj.getString()->getCString());
+    {
+        GString *str = obj.getString();
+        gsize bytes_read = 0;
+        gsize bytes_written = 0;
+        GError *err = NULL;
+
+        // check if BOM exists
+        if (str->getLength() >= 2 &&
+            str->getChar(0) == '\xfe' && str->getChar(1) == '\xff')
+        {
+            // UTF16 big-endian
+            value = g_convert(str->getCString() + 2, str->getLength() - 2,
+                    "UTF-8", "UTF-16BE", &bytes_read, &bytes_written, &err);
+        }
+        else
+        {
+            // assume it's latin1
+            value = g_convert(str->getCString(), str->getLength(),
+                    "UTF-8", "ISO-8859-1", &bytes_read, &bytes_written, &err);
+        }
+        if (err)
+            g_warning("error in conversion: %s", err->message);
+        g_debug("pdf info field %s = %s", key, value);
+    }
 
     info.free();
     obj.free();
@@ -2808,4 +2851,21 @@ pdf_viewer_is_rendering() {
 	return IS_RENDERING( priv );
 }
 
+void
+pdf_viewer_cancel_if_render() {
+    OSSO_LOG_DEBUG(__FUNCTION__);
+
+    /* pdf_viewer_cancel_if_render is called from gtk signal handler, which
+     * locks gdk. We need to unlock it until rendering thread has finished */
+    gdk_threads_leave(); 
+    DUNLOCKED(gdk);
+
+    /* cancel rendering thread */
+    cancel_if_render();
+
+    DTRY(gdk);
+    gdk_threads_enter();
+    DLOCKED(gdk);
+}
+
 /* EOF */
