@@ -98,12 +98,19 @@ pdf_mime_open_cb(ComappSystemData * cas)
 {
     AppUIData *app_ui_data = ((AppData *) cas->user_data)->app_ui_data;
 
-    /* set startup mode */
-    app_ui_data->app_data->mode = STARTUP_MODE_URI_REQUEST;
-
     /* bringing front the application on MIME_OPEN */
     gtk_window_present(GTK_WINDOW(app_ui_data->app_view));
 
+    if (app_ui_data->save_dialog_opened)
+    {
+        /* if save dialog was opened before mime open was called
+         * don't load the file just top the application */
+        return;
+    }
+
+    /* set startup mode */
+    app_ui_data->app_data->mode = STARTUP_MODE_URI_REQUEST;
+
     if (app_ui_data->details_dialog)
     {
         gtk_dialog_response(GTK_DIALOG(app_ui_data->details_dialog),
@@ -116,6 +123,12 @@ pdf_mime_open_cb(ComappSystemData * cas)
                             GTK_RESPONSE_CANCEL);
     }
 
+    if (app_ui_data->opensave_dialog)
+    {
+        gtk_dialog_response(GTK_DIALOG(app_ui_data->opensave_dialog),
+                            GTK_RESPONSE_CANCEL);
+    }
+
     /* attempt to open document */
     ui_open_document(app_ui_data, cas->mime_open_uri, NULL);
 }
