@@ -31,7 +31,6 @@
 #include <hildon/hildon-program.h>
 #include <hildon/hildon-banner.h>
 #include <gdk/gdkkeysyms.h>
-#include <ossoemailinterface.h>
 #include <conbtdialogs-dbus.h>
 #include <dbus/dbus-glib.h>
 
@@ -88,7 +87,9 @@ on_document_save(GtkAction * action, gpointer user_data)
     if( app_ui_data->app_data->state != PDF_VIEWER_STATE_LOADED ) {
     	ui_show_banner( NULL, _("pdfv_ib_menu_not_available") );
     } else {
+        app_ui_data->save_dialog_opened = TRUE;
 	    ui_run_file_chooser(app_ui_data, GTK_FILE_CHOOSER_ACTION_SAVE);
+        app_ui_data->save_dialog_opened = FALSE;
 	}
 }
 
@@ -940,6 +941,10 @@ on_delete_event(GtkWidget * widget, GdkEvent * event, gpointer user_data)
         app_ui_data->close_called = TRUE;
         g_return_val_if_fail(app_ui_data != NULL, FALSE);
 
+        /* wait for finishing rendering thread */
+        TDB("call pdf_viewer_cancel_if_render\n");
+        pdf_viewer_cancel_if_render();
+
         destroy_state_data(app_ui_data->app_data);
         retval = TRUE;
         gtk_main_quit();
