@@ -78,6 +78,7 @@
 
 #define debug(x) x
 
+
 /*******************************************************************************
  **** Private data
  **/
@@ -113,6 +114,16 @@ static HildonFileSystemModel *get_file_system_model(GtkWidget * ref_widget);
 
 static gboolean ui_arrow_hide_on_idle(gpointer data);
 
+/* bug: 87157 STARTS */
+static
+gboolean encrypt_file_banner(gpointer data)
+{
+	gchar *str = (gchar *)data;
+        ui_show_banner(NULL, str);
+	return FALSE;
+}
+/* bug: 87157 ENDS */
+
 
 /*******************************************************************************
  **** Public Functions
@@ -123,14 +134,6 @@ static gboolean ui_arrow_hide_on_idle(gpointer data);
 void
 ui_close_all_banners(AppData * app_data )
 {
-	/* Don't close saving banner as user can do actions while file is saved
-    if (app_data->app_ui_data->saving_banner)
-    {
-        gtk_widget_destroy(app_data->app_ui_data->saving_banner);
-        app_data->app_ui_data->saving_banner = NULL;
-    }
-    */
-    
     if (app_data->app_ui_data->show_images_banner)
     {
         gtk_widget_destroy(app_data->app_ui_data->show_images_banner);
@@ -402,18 +405,15 @@ ui_run_file_chooser(AppUIData * app_ui_data, GtkFileChooserAction action)
 
     gchar *uri_str = NULL;
 
+    if (app_ui_data->last_uri)
     {
-        GnomeVFSURI *uri = pdf_viewer_get_uri();
-        if (uri)
-        {
-            uri_str = gnome_vfs_uri_to_string(uri, GNOME_VFS_URI_HIDE_NONE);
-        }
-        else
-        {
-            uri_str =
-                g_build_filename(g_get_home_dir(),
-                                 SETTINGS_FACTORY_DEFAULT_FOLDER, NULL );
-        }
+        uri_str = g_strdup(app_ui_data->last_uri);
+    }
+    else
+    {
+        uri_str =
+            g_build_filename(g_get_home_dir(),
+                             SETTINGS_FACTORY_DEFAULT_FOLDER, NULL );
     }
 
 	/* Replace unnormal paths with default path (move this to libcomapp?) */
@@ -457,6 +457,7 @@ ui_run_file_chooser(AppUIData * app_ui_data, GtkFileChooserAction action)
         gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(cos->fc_dialog), filter);
     }
 
+    app_ui_data->opensave_dialog = cos->fc_dialog;
     comapp_fc_result res = comapp_opensave_run(cos);
 
     gchar *filename = NULL;
@@ -479,6 +480,10 @@ ui_run_file_chooser(AppUIData * app_ui_data, GtkFileChooserAction action)
             {
                 PDFViewerResult result = pdf_viewer_save(filename);
 
+                if (app_ui_data->last_uri)
+                    g_free (app_ui_data->last_uri);
+                app_ui_data->last_uri = g_strdup(filename);
+
                 if (result == RESULT_SAVE_OK)
                 {
                     comapp_recent_add_new_item(COMAPP_RECENT
@@ -498,6 +503,7 @@ ui_run_file_chooser(AppUIData * app_ui_data, GtkFileChooserAction action)
 
     g_free(uri_str);
     g_object_unref(cos->model);
+    app_ui_data->opensave_dialog = NULL;
 
 }
 
@@ -648,14 +654,16 @@ _ui_open_document(AppUIData * app_ui_data,
         if (retries == 0)
         {
             /* first round, display an infobanner that document is encrypted */
-            ui_show_banner(GTK_WIDGET(app_ui_data->app_view),
-                           _("pdfv_ib_encrypted_file"));
+	    /* bug: 87157 STARTS */
+	    g_idle_add(encrypt_file_banner, _("pdfv_ib_encrypted_file"));
+	    /* bug: 87157 ENDS */
         }
         else
         {
             /* incorrect password */
-            ui_show_banner(GTK_WIDGET(app_ui_data->app_view),
-                           _("pdfv_ib_incorrect_password"));
+	    /* bug: 87157 STARTS */
+            g_idle_add(encrypt_file_banner, _("pdfv_ib_incorrect_password"));
+	    /* bug: 87157 ENDS  */
         }
 
     } while (retries++ < MAX_PASSWORD_RETRIES);
@@ -727,26 +735,34 @@ ui_open_document(AppUIData * app_ui_data,
                  const gchar * filename, const gchar * password)
 {
 	
-    tOpenDoc *p = NULL;
-    if (!app_ui_data->copy_from_gw)
+    if (app_ui_data->app_data->state != PDF_VIEWER_STATE_SAVING)
     {
-        p = g_new(tOpenDoc, 1);
-        p->app_ui_data = app_ui_data;
-        p->filename = g_strdup(filename);
-        p->password = g_strdup(password);
-        app_ui_data->open_document_structure = p;
+        tOpenDoc *p = NULL;
+
+        if (app_ui_data->last_uri)
+            g_free (app_ui_data->last_uri);
+        app_ui_data->last_uri = g_strdup(filename);
+
+        if (!app_ui_data->copy_from_gw)
+        {
+            p = g_new(tOpenDoc, 1);
+            p->app_ui_data = app_ui_data;
+            p->filename = g_strdup(filename);
+            p->password = g_strdup(password);
+            app_ui_data->open_document_structure = p;
+        }
+        else
+        {
+            p = app_ui_data->open_document_structure;
+            /* This can't be set null here or passwd protected files over bt will
+               not work
+            app_ui_data->open_document_structure = NULL; */
+        }
+        if (idle_id)
+            g_source_remove(idle_id);
+        idle_id =
+            g_idle_add_full(G_PRIORITY_DEFAULT_IDLE, idle_open, p, idle_delete);
     }
-    else
-    {
-        p = app_ui_data->open_document_structure;
-        /* This can't be set null here or passwd protected files over bt will
-           not work
-        app_ui_data->open_document_structure = NULL; */
-    }
-    if (idle_id)
-        g_source_remove(idle_id);
-    idle_id =
-        g_idle_add_full(G_PRIORITY_DEFAULT_IDLE, idle_open, p, idle_delete);
 }
 
 
@@ -869,34 +885,6 @@ ui_run_switch_page_dialog(AppUIData * app_ui_data)
 
 }
 
-/**
- * Try to get UTF-8 format of string 
- * @param text string to be converted
- * @return new UTF-8 string or NULL is no UTF-8 version can be found
- */
-static gchar * try_to_find_utf8( const gchar *text ) {
-	/* const gchar * charset; */
-	gchar *utf8_text = NULL;
-	
-	/* No string */
-	if( text == NULL ) {
-		return NULL;
-	}
-	
-	/* It's valid */
-	if( g_utf8_validate( text, -1, NULL ) ) {
-		return g_strdup( text );
-	}
-
-	/* Ok, it must be iso latin 1 then */
-	utf8_text = g_convert( text, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
-	if( utf8_text ) {
-		return utf8_text;
-	}
-	
-	/* Oh no, it wasn't */
-	return NULL;
-}
 
 /**
  * Show file details dialog.
@@ -908,22 +896,18 @@ ui_run_details_dialog(AppUIData * app_ui_data)
 {
     GSList *details = NULL;
     ComappDetailItem *cdi = NULL;
-    gchar * stmp;
 
     /* Construct the detail items */
     cdi = g_new0(ComappDetailItem, 1);
     cdi->name = g_strdup_printf("%s: ", _("pdfv_fi_document_details_title"));
-    stmp = pdf_viewer_get_info(DOCUMENT_DETAILS_TITLE);
-    cdi->content = try_to_find_utf8(stmp);
+    cdi->content = pdf_viewer_get_info(DOCUMENT_DETAILS_TITLE);
     if( cdi->content == NULL )
     	cdi->content = g_strdup( " " );
-    g_free( stmp );
     details = g_slist_append(details, cdi);
 
     cdi = g_new0(ComappDetailItem, 1);
     cdi->name = g_strdup_printf("%s: ", _("pdfv_fi_document_details_author"));
-    stmp = pdf_viewer_get_info(DOCUMENT_DETAILS_AUTHOR);
-    cdi->content = try_to_find_utf8(stmp);
+    cdi->content = pdf_viewer_get_info(DOCUMENT_DETAILS_AUTHOR);
     if( cdi->content == NULL )
     	cdi->content = g_strdup( " " );
     details = g_slist_append(details, cdi);
@@ -931,21 +915,17 @@ ui_run_details_dialog(AppUIData * app_ui_data)
     cdi = g_new0(ComappDetailItem, 1);
     cdi->name =
         g_strdup_printf("%s: ", _("pdfv_fi_document_details_subject"));
-    stmp = pdf_viewer_get_info(DOCUMENT_DETAILS_SUBJECT);
-    cdi->content = try_to_find_utf8(stmp);
+    cdi->content = pdf_viewer_get_info(DOCUMENT_DETAILS_SUBJECT);
     if( cdi->content == NULL )
     	cdi->content = g_strdup( " " );
-    g_free( stmp );
     details = g_slist_append(details, cdi);
 
     cdi = g_new0(ComappDetailItem, 1);
     cdi->name =
         g_strdup_printf("%s: ", _("pdfv_fi_document_details_keywords"));
-    stmp = pdf_viewer_get_info(DOCUMENT_DETAILS_KEYWORDS);
-    cdi->content = try_to_find_utf8(stmp);
+    cdi->content = pdf_viewer_get_info(DOCUMENT_DETAILS_KEYWORDS);
     if( cdi->content == NULL )
     	cdi->content = g_strdup( " " );
-    g_free( stmp );
     details = g_slist_append(details, cdi);
 
     /* fill libcomapp's details dialog structure and let it create the dialog 
@@ -2528,17 +2508,4 @@ ui_set_images_mode(AppUIData * app_ui_data, int sign)
     }
 }
 
-/* Clean ui stuff when quiting */
-void
-ui_clean(AppUIData * app_ui_data) {
-
-	/* Save banner might be left behind, but I don't know if it's good to
-	   close application while saving copy of pdf file */
-    if (app_ui_data->saving_banner)
-    {
-        gtk_widget_destroy(app_ui_data->saving_banner);
-        app_ui_data->saving_banner = NULL;
-    }
-}
-
 /* EOF */
