@@ -212,14 +212,6 @@ extern "C" {
     void ui_update(AppUIData * app_ui_data);
     
     
-/**
-   Check that nothing is left behind in UI
-   
-   @param AppUIData
-*/
-	void ui_clean(AppUIData * app_ui_data);    
-	
-	
 /**
    Loads application state
    
