@@ -649,7 +649,14 @@ void SplashOutputDev::startPage(int pageNum, GfxState *state) {
     if (bitmap) {
       delete bitmap;
     }
-    bitmap = new SplashBitmap(w, h, bitmapRowPad, colorMode, bitmapTopDown);
+    // This can fail if there isn't enough memory to create bitmap
+    try {
+    	bitmap = new SplashBitmap(w, h, bitmapRowPad, colorMode, bitmapTopDown);
+    } catch( int e ) {
+    	bitmap = 0;
+    	splash = 0;
+    	throw 0;
+    }
   }
   splash = new Splash(bitmap);
   switch (colorMode) {
@@ -977,7 +984,7 @@ void SplashOutputDev::updateFont(GfxState *state) {
   FoFiTrueType *ff;
   Ref embRef;
   Object refObj, strObj;
-  GString *tmpFileName, *fileName, *substName;
+  GString *tmpFileName, *fileName/*, *substName*/;
   FILE *tmpFile;
   Gushort *codeToGID;
   DisplayFontParam *dfp;
